package echo;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertTrue;

import com.sun.grizzly.http.SelectorThread;
import com.sun.grizzly.standalone.StaticStreamAlgorithm;
import com.sun.grizzly.tcp.Adapter;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class echoTest
{
	private SelectorThread selectorThread;

	@Before
	public void setUp() throws Exception
	{
		// With this synchronous adapter everything works fine.
//		Adapter adapter = new EchoAdapter();
		
		Adapter adapter = new EchoAsyncAdapter();

		selectorThread = new SelectorThread();
		selectorThread.setAlgorithmClassName( StaticStreamAlgorithm.class.getName() );

		selectorThread.setEnableAsyncExecution( false );

		selectorThread.setPort( 8080 );

		selectorThread.setAdapter( adapter );

		selectorThread.setDisplayConfiguration( true );

		try
		{
			selectorThread.listen();
		}
		catch (InstantiationException ex)
		{
			throw new IOException( ex.getMessage() );
		}
	}

	@After
	public void tearDown() throws Exception
	{
		selectorThread.stopEndpoint();
	}

	@Test
	public void postTest() throws Exception
	{
		URL resource = getClass().getResource( "/rfc2068.txt" );
		File file = new File( resource.getFile() );

		FileInputStream in = new FileInputStream( file );
		String fileContent = new String( readAllBytes( in ) );
		String expected = "##Echo_Top\n" + fileContent + "##Echo_Bottom\n";

		PostMethod filePost = null;
		try
		{
			filePost = new PostMethod( "http://localhost:8080/upload" );
			RequestEntity requestEntity = new FileRequestEntity( file );
			filePost.setRequestEntity( requestEntity );

			HttpClient client = new HttpClient();
			client.getHttpConnectionManager().getParams().setConnectionTimeout( 8000 );

			int status = client.executeMethod( filePost );

			assertEquals( HttpStatus.SC_OK, status );

			String result = new String( readAllBytes( filePost.getResponseBodyAsStream() ) );
			assertTrue( expected.equals( result ) );
		}
		finally
		{
			if (filePost != null)
			{
				filePost.releaseConnection();
			}
		}
	}

	/**
	 * Reads all the bytes from the specified stream, and returns a byte array that contain them.
	 */
	private static byte[] readAllBytes( InputStream is ) throws IOException
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		BufferedInputStream bis = new BufferedInputStream( is );

		int i = bis.read();
		while (i >= 0)
		{
			baos.write( i );
			i = bis.read();
		}
		baos.close();
		return baos.toByteArray();
	}
}
